package com.incubator.game.contest;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.util.DateUtils;
import com.incubator.core.util.DataUtil;
import com.incubator.game.GGame;
import com.incubator.game.data.po.ContestPO;
import com.incubator.game.player.Player;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomService;
import com.incubator.game.util.PlayerUtil;
import com.incubator.game.util.ProtoUtil;
import com.incubator.message.proto.CommonProto;
import org.slf4j.Logger;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 官方比赛实体类
 */
public class Contest {

    protected Logger logger = Log4jUtil.getLogger(getClass());

    /** 比赛数据 **/
    public ContestPO data;

    /** 激活标记 **/
    private boolean isActive;

    /** 房间角色 [key:玩家id, value:玩家对象]**/
    public Map<String, Player> actors = new ConcurrentHashMap<>();

    /** 比赛任务主线程 **/
    private ScheduledExecutorService scheduler;

    public Contest() {
        if (this.scheduler == null || this.scheduler.isShutdown()) {
            this.scheduler = Executors.newSingleThreadScheduledExecutor();
        }
    }

    /**
     * 初始化
     *
     * @param roomType
     * @param contestId
     */
    public void init(int roomType, int contestId) {
        // 初始化比赛
        this.data = new ContestPO();
        this.data.contestId = contestId;
        this.data.startTime = System.currentTimeMillis() + (2 * 60 * 1000);
        this.data.endTime = this.data.startTime + (1 * 60 * 60 * 1000);
        this.data.contestType = 1;
        this.data.level = 1;
        this.data.name = "五常大米第一期大奖赛";
        this.data.url = "https://pic.616pic.com/ad_preview/00/05/85/62733634ac95e.jpg-0.jpg";
        this.data.type = roomType;
    }

    /**
     * 开启官方比赛主线程
     */
    public void startContestTask() {
        this.isActive = true;
        this.scheduler.scheduleAtFixedRate(() -> {
            if (this.isActive) {
                this.doUpdate();
            }
        }, 0, 1, TimeUnit.SECONDS);
    }

    /**
     * 比赛主线程逻辑
     */
    public void doUpdate() {
//        logger.debug("检测: 官方比赛主线程执行...");
        switch (this.data.state) {
            case 0:
                logger.debug("检测: 官方比赛主线程执行...等待开始...比赛id : {}, 人数 : {}, 比赛开始时间 : {}",
                        this.data.contestId, this.actors.size(), DateUtils.formatFullDate(new Date(this.data.startTime)));
                // 等待开始
                if (this.checkReadyStart()) {
                    this.data.state = 1;
                    this.data.flag = false;
                    this.data.time = 0;
                }

                // todo 比赛人数不足直接流局结束比赛
//                if (this.data.time >= 120) {
//                    // 广播
//                    for (Player tmPlayer : this.data.playerMap.values()) {
//                        if (tmPlayer != null) {
//                            tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
//                        }
//                    }
//                    this.destroy();
//                }
                break;
            case 1:
                // 比赛开始
                if (!this.data.flag) {
                    //开始
                    if (this.data.time >= 3) {
                        this.doStart();
                        this.data.state = 2;
                        this.data.flag = false;
                        this.data.time = 0;
                        logger.debug("检测: 比赛id:{}, 比赛人数:{}, 状态:比赛开始...", this.data.contestId, this.data.actors.size());
                    }
                }
                break;
            case 2:
                // 进行中
                if (!this.data.flag) {

                } else {

                }
                break;
            case 3:
                // 结束
                if (!this.data.flag) {
                    this.data.flag = true;
                    this.data.time = 0;
                    logger.debug("比赛结束: 比赛id : {}, 比赛人数 : {}", this.data.contestId, this.data.actors.size());
                }
//                else {
//                    // 小于最大局数,切等待状态
//                    if (this.data.curRound < this.data.maxRound) {
//                        this.resetRoom();
//                        this.data.state = 0;
//                        this.data.flag = false;
//                        this.data.time = 0;
//                    } else {
//                        // 120秒未开始直接解散
//                        if (this.data.time >= 120) {
//                            // 广播
//                            for (Player tmPlayer : this.data.playerMap.values()) {
//                                if (tmPlayer != null) {
//                                    tmPlayer.receive(CommonProto.Cmd.DisbandRoom_VALUE, new HashMap<>());
//                                }
//                            }
//                            this.destroy();
//                        }
//                    }
//                }
                break;
        }

        this.data.time += 1;
    }

    /**
     * 检测是否可以开始 默认3秒后比赛开始
     *
     * @return
     */
    public boolean checkReadyStart() {
        Date now = new Date();
        if (!this.isStart(now) || this.data.state != 0) {
            return false;
        }

        // 广播3秒倒计时
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = ProtoUtil.contestInfoToMessage(this);
                data.put("contestState", 1);
//                data.put("time", 3);
//                data.put("contestTime", GGame.APP_TIME + (3 * 1000));
                data.put("contestTime", this.data.startTime);
                logger.debug("比赛准备开始 : 比赛 : {}, 开始时间 : {}, 人数 : {}", this.data.contestId, DateUtils.formatFullDate(new Date(this.data.startTime)), this.actors.size());
                tmPlayer.receive(CommonProto.Cmd.MatchPreparationStart_VALUE, data);
            }
        }

        return true;
    }

    /**
     * 比赛是否可开始
     *
     * @return
     */
    public boolean isStart(Date date) {
        // 时间是否到了
        return date.getTime() >= this.data.startTime - (3 * 1000);
    }

    /**
     * 比赛开始
     */
    public void doStart() {
        // 玩家匹配,分配房间
        Room room = RoomService.getInstance().creatRoom(null, 1);
        if (room == null) {
            logger.info("比赛开始...创建房间出错...比赛 : {}, name : {}", this.data.contestId, this.data.name);
            return;
        }

        // 开启房间主线程
        room.startRoomTask();

        // 推送广播,拉玩家进房间
        for (Player tmPlayer : this.actors.values()) {
            if (tmPlayer == null) {
                continue;
            }
            Map<String, Object> data = ProtoUtil.contestInfoToMessage(this);
            data.put("roomId", room.roomId);
            data.put("roomType", room.type);
            logger.debug("比赛开始 : 比赛 : {}, 玩家 : {}", this.data.contestId, this.actors);
            tmPlayer.receive(CommonProto.Cmd.MatchOfficialStart_VALUE, data);
        }
    }

    /**
     * 比赛重置逻辑
     */
    public void resetContest() {

    }

    /**
     * 重置比赛，用于对象池回收前清理状态
     */
    public void reset() {
        this.isActive = false;
    }

    /**
     * 销毁比赛，释放资源
     */
    public void destroy() {
        this.reset();
        this.scheduler.shutdownNow();
        this.scheduler = null;
        // 重置比赛数据
        this.resetContest();
        // 归还比赛对象
        ContestService.getInstance().releaseContest(this);
        logger.debug("Contest {} has been destroyed.", this.data.contestId);
    }
}
