package com.incubator.game.handler.contest;

import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.contest.Contest;
import com.incubator.game.contest.ContestService;
import com.incubator.game.player.Player;
import com.incubator.game.util.PlayerUtil;
import com.incubator.game.util.ProtoUtil;
import com.incubator.message.proto.CommonProto;

import java.util.HashMap;

/**
 * 加入比赛/报名 请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.MatchSignUpReq_VALUE)
public class JoinContestHandler extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.MatchSignUpRes_VALUE);

        Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
        if (player == null) {
            logger.info("创建房间失败，找不到玩家");
            response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
            response.setMessage("操作失败...");
            return;
        }

        // 判断参数
        int contestId = request.getDataValue("contestId", Integer.class, 0);
        if (contestId <= 0) {
            logger.info("参数错误");
            response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
            response.setMessage("参数错误...");
            return;
        }

        // 比赛是否存在
        Contest contest = ContestService.getInstance().contestMap.getOrDefault(contestId, null);
        if (contest == null) {
            logger.info("比赛不存在或已结束");
            response.setCode(CommonProto.Code.CONTEST_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，比赛不存在或已结束...");
            return;
        }

        // 是否已加入其他比赛
        if (player.data.contestId > 0 && player.data.contestId != contestId) {
            logger.info("不可重复加入其他比赛");
            response.setCode(CommonProto.Code.CONTEST_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，不可重复加入其他比赛...");
            return;
        }

        // todo 扣门票

        // 报名加入比赛
        contest = ContestService.getInstance().joinContest(player, contest.data.contestId);

        // 正常返回
        response.setData(ProtoUtil.contestInfoToMessage(contest));
    }
}
